from time import ticks_ms, time, sleep

from picogo.Robot import Robot
from picogo.ObstacleAvoidance import run_obstacle_avoidance

def mainImpl( robot ) :

    print("Start Lane Tracking ...") 
    
    robot.stop()
    sleep( 1 )
    
    speed = 30
    
    for i in range(100) :
        if  25 < i <= 75:
            robot.move( speed, -speed, False )
        else:
            robot.move( -speed, speed, False )
        pass
    
        robot.trs_calibrate()
    pass

    print( "calibratedMin = ", robot.trs.calibratedMin )
    print( "calibratedMax = ", robot.trs.calibratedMax )
    print( "calibrate done!")
    print()
    
    robot.stop() 
    sleep(1)
    
    robot.disp_info_rects() 

    integral = 0
    last_proportional = 0 
    
    then_ms = ticks_ms()
    
    count = 0
    
    while robot.run_ext_module :
        count += 1
        max_speed = min( 80, 1*robot.speed )
        
        robot.disp_battery()
        robot.disp_motor()
        
        duration = robot.duration
        
        position, sensors = robot.readLine()
        
        now_ms = ticks_ms()
        
        elapsed_ms = now_ms - then_ms
        
        print( f"[{count:05d}] now_ms = {now_ms}, then_ms = {then_ms}, elpased_ms = {elapsed_ms}, position = {position}, sensors = {sensors}" )
        
        # The "proportional" term should be 0 when we are on the line.
        proportional = 1000*position - 2500

        # Compute the derivative (change) and integral (sum) of the position.
        derivative = ( proportional - last_proportional )
        integral += proportional

        # Remember the last position.
        last_proportional = proportional
        
        speed_diff = proportional/30  + derivative*2.5 + integral*0.0001
        ## speed_diff = proportional/30  + derivative*2;  

        speed_diff = max( - max_speed, min( speed_diff, max_speed ) )
        
        if speed_diff < 0 :
            robot.move( max_speed + speed_diff, max_speed )
        else:
            robot.move( max_speed, max_speed - speed_diff )
        pass
    
        then_ms = now_ms
    
        ## sleep( duration ) 
    pass

    robot.stop() 
    robot.disp_logo()

    print( f"Finished running lane tracking." )

pass

def main( robot ) :
    robot.run_ext_module = True
    
    import _thread 
    
    callback = lambda : mainImpl( robot=robot )
    
    _thread.start_new_thread( callback, () )
    
    print( "thread inited" )
pass

if __name__ is '__main__' :
    
    print( "Hello ..." )
        
    robot = Robot()
    main( robot )
    
    duration = 60
    print( f"sleep({duration})" )
    sleep( duration )
    print( f"finished sleep({duration})" )
    
    robot.run_ext_module = False
    
    print( "Good bye!" )
    
pass



from time import sleep 

from picogo.UltraSonic import UltraSonic

if __name__ is '__main__':
    ultraSonic = UltraSonic()
    
    while True:
        dist = ultraSonic.distance()
        print( f"Distance = {dist:6.2f} cm" )
        sleep( 0.1)
    pass
pass



from machine import Pin, SPI
from time import sleep
from random import randint
import framebuf, builtins

from picogo.Battery import Battery
from picogo.UltraSonic import UltraSonic
from picogo.IRSensor import IRSensor
from picogo.TRSensor import TRSensor
from picogo.Motor import Motor

import picogo
    
# ST7789 # 240 x 135
class LCD( framebuf.FrameBuffer ):
    
    # RGC 565 Color Definition
    WHITE  = white  = 0xFFFF
    BLACK  = black  = 0x0000
    GREEN  = green  = 0x001F
    RED    = red    = 0x00F0
    BLUE   = blue   = 0xFF00
    GBLUE  = gblue  = 0X07FF
    YELLOW = yellow = 0xF0FF
    
    def __init__(self):
        self.width = 240
        self.height = 135
        
        self.bg = self.BLACK  # background color
        self.fg = self.WHITE  # foreground color
        
        self.texts = []
        self.text_idx = 0 
        
        self.inited_print = False
        
        self.rst = Pin(12,Pin.OUT)
        self.bl = Pin(13,Pin.OUT)
        self.bl(1)
        
        self.cs = Pin(9,Pin.OUT)
        self.cs(1)
        self.spi = SPI(1)
        self.spi = SPI(1,1000_000)
        self.spi = SPI(1,10000_000,polarity=0, phase=0,sck=Pin(10),mosi=Pin(11),miso=None)
        self.dc = Pin(8,Pin.OUT)
        self.dc(1)
        self.buffer = bytearray(self.height * self.width * 2)
        
        super().__init__(self.buffer, self.width, self.height, framebuf.RGB565)
        
        ## cod by sunabove ##
        self.init_display()
        
        self.fill( self.bg ) 
        
        x = 0
        y = 0
        rc = row_count = 6
        
        width = self.width
        height = self.height
        
        self.rect( x, y, width, height - 1, self.fg, False )
        
        m = 4
        
        rects = self.rects = []
        
        x = m
        w = width - 2*m
        h = int( (height - (rc + 2)*m - 1)/rc )
        
        for idx in range( rc ) :
            y = idx*h + (idx + 1)*m
            rect = [ x, y, w, h, m ]
            rects.append( rect )
        pass
    
    pass ### __init__

    def disp_info_rects( self ) : 
        x = 0
        y = 0 
        
        width = self.width
        height = self.height
        
        self.rect( x, y, width, height - 1, self.bg, True )
        self.rect( x, y, width, height - 1, self.fg, False )
        
        rects = self.rects
        
        for r in rects :
            self.rect( r[0], r[1], r[2], r[3], self.fg, False )
        pass
    
    pass 
        
    def write_cmd(self, cmd):
        self.cs(1)
        self.dc(0)
        self.cs(0)
        self.spi.write(bytearray([cmd]))
        self.cs(1)
    pass

    def write_data(self, buf):
        self.cs(1)
        self.dc(1)
        self.cs(0)
        self.spi.write(bytearray([buf]))
        self.cs(1)
    pass

    def init_display(self):
        """Initialize dispaly"""
        self.rst(1)
        self.rst(0)
        self.rst(1)
        
        self.write_cmd(0x36)
        self.write_data(0x70)

        self.write_cmd(0x3A) 
        self.write_data(0x05)

        self.write_cmd(0xB2)
        self.write_data(0x0C)
        self.write_data(0x0C)
        self.write_data(0x00)
        self.write_data(0x33)
        self.write_data(0x33)

        self.write_cmd(0xB7)
        self.write_data(0x35) 

        self.write_cmd(0xBB)
        self.write_data(0x19)

        self.write_cmd(0xC0)
        self.write_data(0x2C)

        self.write_cmd(0xC2)
        self.write_data(0x01)

        self.write_cmd(0xC3)
        self.write_data(0x12)   

        self.write_cmd(0xC4)
        self.write_data(0x20)

        self.write_cmd(0xC6)
        self.write_data(0x0F) 

        self.write_cmd(0xD0)
        self.write_data(0xA4)
        self.write_data(0xA1)

        self.write_cmd(0xE0)
        self.write_data(0xD0)
        self.write_data(0x04)
        self.write_data(0x0D)
        self.write_data(0x11)
        self.write_data(0x13)
        self.write_data(0x2B)
        self.write_data(0x3F)
        self.write_data(0x54)
        self.write_data(0x4C)
        self.write_data(0x18)
        self.write_data(0x0D)
        self.write_data(0x0B)
        self.write_data(0x1F)
        self.write_data(0x23)

        self.write_cmd(0xE1)
        self.write_data(0xD0)
        self.write_data(0x04)
        self.write_data(0x0C)
        self.write_data(0x11)
        self.write_data(0x13)
        self.write_data(0x2C)
        self.write_data(0x3F)
        self.write_data(0x44)
        self.write_data(0x51)
        self.write_data(0x2F)
        self.write_data(0x1F)
        self.write_data(0x1F)
        self.write_data(0x20)
        self.write_data(0x23)
        
        self.write_cmd(0x21)
        self.write_cmd(0x11)
        self.write_cmd(0x29)
    pass

    def show(self):
        self.write_cmd(0x2A)
        self.write_data(0x00)
        self.write_data(0x28)
        self.write_data(0x01)
        self.write_data(0x17)
        
        self.write_cmd(0x2B)
        self.write_data(0x00)
        self.write_data(0x35)
        self.write_data(0x00)
        self.write_data(0xBB)
        
        self.write_cmd(0x2C)
        
        self.cs(1)
        self.dc(1)
        self.cs(0)
        self.spi.write(self.buffer)
        self.cs(1)
    pass

    def flush( self ) :
        self.show()
    pass # flush

    def disp_text(self, text, x, y, fg=None, bg=None, flush=True) :
        if bg is None : bg = self.bg
        if fg is None : fg = self.fg
        
        width = self.width
        height = self.height
        
        self.rect( 2, 2, width -5, height -5, bg, True )
        
        for t in text.split( "\n" ) : 
            self.text( t, x, y, fg )
            y += 15
        pass
    
        flush and self.flush()
    pass # disp_text

    def print(self, *args, **kwargs) :
        width = self.width; height = self.height
        line_height = 15
        
        x = 0 ; y = 0
        
        m = 4
        w = width - 1
        h = int( (height - m - 1)/2 )
        y = h + m
        
        self.rect( x, y, w, h, self.bg, True )
        self.rect( x, y, w, h, self.fg, False )
        
        fg = None
        if "c" in kwargs :
            fg = kwargs.pop( "c" )
        elif "fg" in kwargs :
            fg = kwargs.pop( "fg" )
        else :
            fg = self.fg
        pass
    
        texts = "".join( args )
        
        for text in texts.split( "\n" ) :
            self.texts.append( [ text, fg, self.text_idx ] )
            self.text_idx += 1
        pass
    
        while len( self.texts ) > 8 :
            self.texts.pop( 0 )
        pass
    
        x = 10
        y += line_height 
        
        line_count = int( (height - y)/line_height )
        
        #print( f"line count = {line_count}" )
        
        for text in self.texts[ - line_count :  ] :
            t = text[ 0 ]
            fg = text[ 1 ]
            text_idx = text[ 2 ]
            
            t = f"[{text_idx + 1:2d}] {t}"
            self.text( t, x, y, fg )
            y += line_height
            
            #builtins.print( text )
        pass
        
        if False :
            builtins.print( "LCD : ", end="" )
            builtins.print( *args, **kwargs )
        pass
    pass # print

    def disp_init( self, flush=False ) :
        self.disp_battery()
        self.disp_ultra_sonic()
        self.disp_ir_sensor()
        self.disp_tr_sensor()
        self.disp_motor( flush=True )
    pass

    def disp_battery( self, values = [0, 0, 0], verbose=False, flush=False ) : # 배터리 잔량 표시 
        
        if isinstance( values , Battery ) :
            values = values.read()
        pass 
        
        temp, voltage, percent = values
        
        color = LCD.GREEN
        if percent < 33 :
            color = LCD.RED
        elif percent < 66 :
            color = LCD.BLUE
        else :
            color = LCD.GREEN
        pass
    
        fg = self.fg
        
        x, y, w, h, m = self.rects[0]
        
        self.rect( x, y, w, h, LCD.GBLUE, True )
        self.rect( x, y, int( w*percent/100 ), h, color, True )
        self.rect( x, y, w, h, fg, False )
        
        if False : self.text( f"{int(percent):3d} %", int(x + w/2 - 25), int( y + 5), LCD.WHITE )
        
        if flush : self.flush()
        
        if verbose : print( f"Temperature = {temp:.3f} °C, Voltage = {voltage:.3f} V, Percent = {percent:.2} %" )
        
    pass # disp_battery

    def disp_ultra_sonic( self, dist = 0 , verbose = 0, flush=0 ) :  # 초음파 센서 거리 표시 
        
        if isinstance( dist, UltraSonic ) :
            dist = dist.get_obstacle_distance()
        pass 
        
        color = LCD.GREEN
        
        max_dist = 40
        
        if dist <= 10 :
            color = 0xFFF0            
        elif dist <= 20 :
            color = LCD.RED
        elif dist <= 30 :
            color = LCD.BLUE
        else :
            color = LCD.GREEN
        pass
    
        fg = self.fg
        bg = self.bg
        
        x, y, w, h, m = self.rects[1]
        
        value_width = int( w*min(max_dist, dist)/max_dist )
        
        self.rect( x, y, w, h, LCD.GBLUE, True )
        
        if False :
            self.rect( x, y, value_width, h, color, True )
        else :
            cw = cell_width = int(w/10)
            
            for idx in range( int(value_width/cw + 0.5) ):
                c = color
                self.rect( x + idx*cw, y, cw - m, h, c, True )
            pass
        pass
        
        self.rect( x, y, w, h, fg, False )
        
        if flush : self.flush()
    
    pass # disp_ultra_sonic

    def disp_ir_sensor( self, blocks = [0, 0], flush = 0 ) :
        
        if isinstance( blocks, IRSensor ) :
            blocks = blocks.read_blocks()
        pass
        
        x, y, w, h, m = self.rects[2]
        
        bg = self.bg
        fg = self.fg
        
        self.rect( x, y, w, h, bg, True )
        self.rect( x, y, w, h, fg, False )
        
        x += m
        y += 1
        w = int( (w - 3*m)/2 )
        h = h - 2
        
        for idx, block in enumerate( blocks ) :
            c = LCD.RED if block else LCD.GREEN
            self.rect( x + idx*(w +m) , y, w, h, c, True )
            self.rect( x + idx*(w +m) , y, w, h, fg, False )
        pass
    
        if flush : self.flush()
        
    pass # disp_infrared_sensor

    def disp_tr_sensor( self, position = -1, sensors = [0]*5, flush = 0 ) :
        
        # draw sensor rectagnel 
        x, y, w, h, m = self.rects[3]
        h = 2*h + m
        
        bg = self.bg
        fg = self.fg
        
        self.rect( x, y, w, h, bg, True )
        self.rect( x, y, w, h, fg, False )
        
        sensors_len = len( sensors )
        
        w1 = (w - m*sensors_len - m)/sensors_len
        
        for idx, sensor in enumerate( sensors ) :
            sensor = min( sensor, 1_000 )
            sensor = max( sensor, 100 )
            
            h1 = (h - 2*m)*sensor/1_000
            x1 = x + idx*(w1 + m) + m
            y1 = y + m + (h - 2*m - h1)
            
            color = LCD.YELLOW
            
            self.rect( int(x1), int(y1), int(w1), int(h1), color, True )
            self.rect( int(x1), int(y + m), int(w1), int(h - 2*m), fg, False )
            
            #print( f"x1 = {x1}, y1 = {y1}, w1 = {w1}, h1 = {h1}" )
        pass
    
        # draw position circle
        we = h/4
        he = we
        
        xe = x + m + 3 + we/2
        
        pos = position
        
        if pos < 0 :
            xe = x + m + 3 + we/2
        elif pos > sensors_len - 1 : 
            xe = x + w - we - m - 4
        else :
            xe = x + w*(pos + 0.5)/sensors_len - we/2
        pass
    
        ye = y + h/2 
        
        self.ellipse( int(xe), int(ye), int(we), int(he), LCD.BLUE, True )
        self.ellipse( int(xe), int(ye), int(we), int(he), fg, False )
    
        if flush : self.flush()
    pass # disp_tr_sensor

    def disp_motor( self, speeds = [ 0, 0 ], flush=False ) :
        
        if isinstance( speeds, Motor ) :
            speeds = speeds.speeds
        pass
    
        x, y, w, h, m = self.rects[5]
        
        bg = self.bg
        fg = self.fg
        
        self.rect( int(x), int(y), int(w), int(h), bg, True )
        
        w_0 = (w - m)/2
        
        fg = self.green
        
        for idx, speed in enumerate( speeds ):
            c = self.green if speed > 0 else self.red
            x_0 = x + idx*(w_0 + m)
            
            w_1 = w_0*abs( speed )/100
            x_1 = x + idx*(w_0 + m)
            if idx ==0 :
                x_1 = x_1 + w_0 - w_1
            pass
            
            self.rect( int(x_1), int(y), int(w_1), int(h), c, True )
            self.rect( int(x_0), int(y), int(w_0), int(h), LCD.white, False )
        pass
        
        if flush : self.flush()
    pass # disp_motors

    def disp_full_text(self, text, fg=None, bg=None, flush=True) :
        if bg is None : bg = self.bg
        if fg is None : fg = self.fg
        
        width = self.width
        height = self.height
        
        m = 2
        
        self.rect( m, m, width - 2*m -1, height - 2*m -1, bg, True )
        
        if isinstance( text, str ) : 
            text = text.split( "\n" )
        pass
    
        text_len = len( text )
        
        x = 0
        y = height/2
        cw = ch = 8
        
        for i, t in enumerate( text ) :
            x = (width - cw*len(t))/2
            y = height/2 - 2*ch*( int(text_len/2) - i )
            
            self.text( t, int( x ), int( y), fg ) 
        pass
    
        flush and self.flush()
    pass # disp_full_text

    def color(self, R, G, B): # Convert RGB888 to RGB565
        return (((G&0b00011100)<<3) +((B&0b11111000)>>3)<<8) + (R&0b11111000)+((G&0b11100000)>>5)
    pass

    def disp_logo(self, fg=None, bg=None, flush=True) : 
        print( "disp_logo" )
        
        self.disp_logo_text( fg=fg, bg=bg, flush=flush)
    pass

    def disp_logo_text(self, fg=None, bg=None, flush=True) :
        from .LogoText import logo_text

        self.disp_full_text( logo_text, fg=fg, bg=bg, flush=flush )
    
        del logo_text
    pass

    def disp_version( self ) :
        print( "disp_version" )

        version = picogo.__version__
        
        self.disp_full_number( version, flush=True )
    pass

    def disp_logo_image(self, logo_image, flush=True) : 
        
        bg = self.bg
        fg = self.fg 
        
        width = self.width
        height = self.height
        
        m = 0
        
        self.rect( m, m, width - 2*m -1, height - 2*m -1, bg, True )
        
        cw = 8
        ch = 16
        m = 2
        
        x0 = 2*m
        y0 = 2*m
        
        x = x0
        y = y0
        
        for t in logo_image :
            
            if t != '\n' :
                #print( f"x = {x}, y = {y}, t = {t}" )            
            
                if t == '∙' : t = ''
                self.text( t, int(x), int(y), fg )
                x += cw
            else :
                #print( f"new line encountered" );
                y += ch
                x = x0
            pass
        pass
    
        flush and self.flush() 
    pass

    def disp_full_number( self, number=0, fg=None, bg=None, flush=True) :
        debug = False
        
        from picogo.TextNumber import numbers
        
        number = int(number)
        number = f"{number:d}"
        
        texts = numbers[0] 
        texts = texts.split( "\n" )
        
        if debug :
            for t in texts :
                print( f"***{t}***" )
            pass
        pass
    
        row = len( texts )        
        texts = [ '' ] * row
        
        debug and print( f"row = {row}" )
        
        c = len( number )
        if c >= 4 or c < 1 :
            c = 0
        else :
            c = 4 - c
        pass
    
        c = " " * c
        
        for n in number :
            n = int( n )
            
            debug and print( f"n = {n}" )
            
            n = numbers[ n ]
            n = n.split( "\n" )
            
            for i, (t1, t2) in enumerate( zip( texts, n ) ) :
                texts[i] = t1 + c + t2
            pass            
        pass
    
        if debug :
            for t in texts :
                print( f"***{t}***" )
            pass
        pass
    
        self.disp_full_text( texts, fg=fg, bg=bg, flush=flush)
    
        del numbers
    pass

    def disp_pairing_code( self, fg=None, bg=None, flush=True) :
        debug = False
        
        if bg is None : bg = self.black
        if fg is None : fg = self.yellow
        
        import random as my_random
        
        number = my_random.randint( 1000, 9999 )
        
        texts = f"{number}"
        
        self.disp_full_number( texts, fg=fg, bg=bg, flush=flush)
        
        del my_random
        
        return texts
    pass

pass ## -- class LCD

if __name__== '__main__' :
    # LCDPrintTest.py
    print( "Hello..." )
    
    lcd = LCD()
    lcd.flush() 
    
    pair_code = lcd.disp_pairing_code( flush=True )
    
    print( f"pari code = {pair_code}" )
    
    sleep( 1 )
elif __name__== '__main__' :
    # LCDPrintTest.py
    print( "Hello..." )
    
    lcd = LCD()
    lcd.flush() 
    
    lcd.disp_full_number( 7890, flush=True )
    
    sleep( 1 )
elif __name__== '__main__' :
    # LCDPrintTest.py
    print( "Hello..." )
    
    lcd = LCD()
    lcd.flush()
    
    sleep( 1 )
    
    lcd.disp_logo( flush=True )
elif __name__== '__main__' :
    # display tr sensor
    motor = Motor() 
    lcd = LCD()
    trs = TRSensor()
    
    lcd.disp_init(flush=1)
    
    # calibration
    motor.stop()
    
    sleep(3)
    
    for i in range(100) :
        if  25 < i <= 75:
            motor.set_motor( 30, -30, False )
        else:
            motor.set_motor(-30, 30, False )
        pass
    
        lcd.disp_motor( motor, flush=True )
    
        trs.calibrate()
    pass

    motor.stop()
    
    idx = 0 
    while True:
        idx += 1
        
        position, sensors = trs.readLine()        
        lcd.disp_tr_sensor( position, sensors, flush=True )
        
        #speeds = [ randint( -100, 100 ), randint( -100, 100 ) ]
        
        lcd.disp_motor( motor, flush=True )
        
        print( f"[{idx:4d}] position = {position:+.2f}, sensors = {sensors}" )
    
        sleep( 1 )
    pass
    
elif __name__== '__main__' :
    # display infrared sensor
    
    lcd = LCD()
    irSensor = IRSensor()
    
    lcd.disp_init() 
    
    idx = 0 
    while 1 :
        idx += 1
        blocks = irSensor.read_blocks()
        lcd.disp_ir_sensor( blocks, flush=True ) 
        
        print( f"[{idx:4d}] InfraRed LEFT: {blocks[0]}, RIGHT: {blocks[1]}" )
        
        sleep( 0.1)
    pass
elif __name__== '__main__' :
    # display ultra sonic
    
    lcd = LCD()
    ultraSonic = UltraSonic()
    
    lcd.disp_init()
    
    while 1 :
        dist = ultraSonic.obstacle_distance()
        lcd.disp_ultra_sonic( dist )
        lcd.flush()
        
        print( f"Distance = {dist:6.2f} cm" )
        sleep( 0.1)
    pass
elif __name__== '__main__' :
    # dispaly battery
    
    lcd = LCD()    
    battery = Battery()
    
    idx = 0
    while True :
        idx += 1
        
        values = [ temp, voltage, percent ] = battery.read()
        
        #values = [ temp,  voltage, randint( 0, 100 ) ] 
        
        lcd.disp_battery( values, 1 )
        lcd.flush()
        
        sleep( 1 )
    pass
pass




from picogo.LCD import LCD 
        
if __name__=='__main__':
    lcd = LCD()
    
    lcd.fill( LCD.BLACK )
    
    lcd.text( "Raspberry Pi Pico", 10, 5, LCD.BLUE )
    lcd.text( "PicoGo", 10, 15, LCD.WHITE )
    lcd.text( "SkySLAM", 10, 25, 0x07E0 )
    
    lcd.show() 
pass


from random import randint
from time import sleep

from picogo.LCD import LCD
from picogo.Battery import Battery

if __name__=='__main__': 
    
    lcd = LCD()
    battery = Battery()
    
    lcd.disp_init()
    
    while True :
        
        lcd.disp_battery( battery )
        lcd.show()
        
        sleep( 1 )
    pass 
pass


import machine , ujson, time
from machine import Pin, Timer
from time import sleep

from picogo.CurrentTime import curr_time_mili
from picogo.Motor import Motor
from picogo.RGBLed import RGBLed
from picogo.LCD import LCD
from picogo.UltraSonic import UltraSonic
from picogo.IRSensor import IRSensor
from picogo.TRSensor import TRSensor
from picogo.Battery import Battery

print( "Import Robot ..." )

class Robot : 
    
    LOW_SPEED = low_speed  = 30
    MED_SPEED = med_speed  = 50
    HIGHT_SPEED = high_speed = 80
    
    def __init__(self):
        self.speed = self.low_speed
        
        # paramenter for obstacle avoidance
        self.run_ext_module = False
        self.duration = 0.001
        self.max_dist = 18
        
        # input devices
        self.ultraSonic = UltraSonic()
        self.irSensor = IRSensor()
        self.trs = TRSensor()
        self.battery = Battery() 
        
        # output devices
        self.rgbLed = RGBLed()
        self.buzzer = machine.Pin(4, Pin.OUT)
        self.led = machine.Pin(25, Pin.OUT)
        self.lcd = LCD()
        self.motor = Motor()
        
        # bluetooth uart        
        self.uart = machine.UART(0, 115200)
        
        self.init_robot()
    pass ## -- __init__

    def init_robot( self ):
        print( "init_robot")
        
        lcd = self.lcd
        
        duration = 1
        
        lcd.disp_logo()
        
        self.ledToggle()        
        
        self.beepOnOff( repeat = 1, period = 1.2 )
        sleep( duration )
        
        lcd.disp_version()        
        sleep( duration )
        
        lcd.disp_logo()        
        sleep( duration )
        
        if False :
            lcd.disp_full_text( "PicoRun\nSkySLAM\nVer 1.0", flush=True )    
            sleep( duration )
        pass
        
        print( "done init robot." )
    pass ## -- init_robot

    def disp_info_rects( self ) :
        print( f"disp_info_rects" )
        
        lcd = self.lcd
        lcd.disp_info_rects()
    pass

    def disp_logo( self, flush=1 ) :
        self.lcd.disp_logo(flush=flush)
    pass

    def disp_battery( self, flush=0 ):
        self.lcd.disp_battery( self.battery, flush=flush )
    pass

    def disp_speeds( self, flush=0 ) :
        self.diso_motor( flush=flush )
    pass

    def disp_motor( self, flush=0 ) :
        self.lcd.disp_motor( self.motor, flush=flush )
    pass 

    def read_blocks(self):
        blocks = self.irSensor.read_blocks()
        
        lcd = self.lcd
        lcd.disp_ir_sensor( blocks, flush=True )
        
        return blocks
    pass

    def trs_calibrate( self ) :
        return self.trs.calibrate()
    pass

    def readLine(self, white_line = 0):
        line = [ position, sensors ] = self.trs.readLine( white_line = white_line )
        
        lcd = self.lcd
        lcd.disp_tr_sensor( position, sensors, flush=True )
        
        return line
    pass

    def obstacle_distance( self ) :
        dist = self.ultraSonic.obstacle_distance()
        
        lcd = self.lcd
        lcd.disp_ultra_sonic( dist, flush=True )
        
        return dist;
    pass 

    def beepOnOff(self, repeat = 1, period = 0.5, verbose=False ) :
        print( "beepOnOff" )
        
        buzzer = self.buzzer
        
        repeat = max( 1, repeat ) 
    
        class BuzzerOnOff :
            def __init__( self, buzzer, max_count = 10, verbose=False ) :
                self.buzzer = buzzer
                self.count = 0
                self.max_count = max_count
                self.verbose = verbose
            pass
                
            def toggle( self, timer ):
                buzzer = self.buzzer
                
                if self.count < self.max_count :
                    if self.verbose : print( f"[{self.count}] toggle" )
                    
                    buzzer.value( int((self.count + 1)%2) )
                else :
                    if self.verbose : print( "timer deinit" )
                    
                    buzzer.value( 0 )
                    
                    timer.deinit()
                pass
            
                self.count += 1
            pass
        pass

        buzzerOnOff = BuzzerOnOff( buzzer=buzzer, max_count = 2*repeat, verbose=verbose )
        
        timer = Timer()
        
        freq = 1/period
        
        timer.init( freq=freq, mode=Timer.PERIODIC, callback=buzzerOnOff.toggle )
    
        print( "done. beepOnOff" )
    pass ## -- beepOnOff

    def beepOnOffOld(self, repeat = 1, period = 0.5 ) :
        print( "beepOnOff" )
        
        buzzer = self.buzzer
        
        repeat = max( 1, repeat )
        
        for i in range( repeat*2 ) :
            buzzer.value( (i+1) % 2 )
            sleep( period )
        pass
    
        print( "done. beepOnOff" )
    pass ## -- beepOnOff

    def ledToggle(self, repeat = 10, period = 0.5, verbose=False ) :
        print( f"ledToggle( repeat={repeat}, period={period} )" )
        
        led = self.led
        
        class LedBlink :
            def __init__( self, led, max_count = 10, verbose=False ) :
                self.led = led
                self.count = 0
                self.max_count = max_count
                self.verbose = verbose
            pass
                
            def blink( self, timer ):
                if self.count < self.max_count :
                    if self.verbose : print( f"[{self.count}] toggle" )
                    
                    self.led.toggle()
                else :
                    if self.verbose : print( "timer deinit" )
                    
                    timer.deinit()
                pass
            
                self.count += 1
            pass
        pass

        ledBlink = LedBlink( led=led, max_count = 2*repeat, verbose=verbose )
        timer = Timer()
        freq = 1/period
        
        timer.init( freq=freq, mode=Timer.PERIODIC, callback=ledBlink.blink )
    pass # -- ledToggle

    def forward(self, speed=None, verbose=False ):
        if speed is None : speed = Robot.LOW_SPEED
        
        self.motor.forward( speed, verbose )
    pass

    def backward(self, speed=None, verbose=False ):
        if speed is None : speed = Robot.LOW_SPEED
        
        self.motor.backward( speed, verbose )
    pass

    def left(self, speed=None, verbose=False ):
        if speed is None : speed = Robot.LOW_SPEED
        
        self.motor.left( speed, verbose )
    pass

    def right(self, speed=None, verbose=False ):
        if speed is None : speed = Robot.LOW_SPEED
        
        self.motor.right( speed, verbose )
    pass

    def stop(self, verbose=False):
        self.motor.stop( verbose )
    pass

    def move(self, left, right, verbose=False):
        if left is None : left = Robot.LOW_SPEED
        if right is None : right = - left
        
        self.motor.move( left, right, verbose )
    pass

    def set_motor(self, left, right, verbose=False):
        self.motor.set_motor( lef, right, verbose )
    pass

pass ## -- class Robot

if __name__ is '__main__' : 
    
    robot = Robot()
    
    robot.forward()
    sleep( 2 )
    
    robot.backward()
    sleep( 2 )
    
    robot.left()
    sleep( 2 )
    
    robot.right()
    sleep( 2 )
    
    robot.stop()
    
pass


__name__ = "picogo"
__package__ = "picogo"
__version__ = '102'
__author__ = "SkySLAM Co., Ltd."

print( "picogo __init__.py executed." ) 

from .Robot import (
    Robot  
)

print( "Done. picogo __init__.py executed." ) 


from time import sleep

from picogo.TRSensor import TRSensor
from picogo.Motor import Motor
from picogo.LCD import LCD

if __name__ == '__main__' :
    
    print("TRSensor Test Program ...")
    
    lcd = LCD() 
    lcd.disp_text( f"LineTracking", 50, 60 )
    
    motor = Motor()
    trs = TRSensor()
    
    motor.stop()
    
    sleep(3)
    
    for i in range(100) :
        if  25 < i <= 75:
            motor.set_motor( 30, -30, False )
        else:
            motor.set_motor( -30, 30, False )
        pass
    
        trs.calibrate()
    pass

    motor.stop()

    print( "calibratedMin = ", trs.calibratedMin )
    print( "calibratedMax = ", trs.calibratedMax )
    print( "calibrate done!")
    print()
    
    sleep(3)

    numSensors = trs.numSensors
    center_position = trs.center_position()
    
    max_power = 15 

    while True:
        position, sensors = trs.readLine()
        
        print( "position = ", position, ", Sensors = ", sensors )        
        
        pos_diff = position - center_position
        
        lcd.disp_text( f"pos  = {position:4.3f}\ndiff = {pos_diff:6.3f}", 50, 60 )
        
        if True :
            power_diff = max_power*pos_diff/center_position
            
            motor.move( max_power + power_diff, max_power - power_diff )
        elif abs( pos_diff ) < numSensors/4 :
            motor.forward( max_power )
        elif pos_diff > 0 :
            power = max_power*abs(pos_diff)/center_position
            
            motor.right( power )
        elif pos_diff < 0 :
            power = max_power*abs(pos_diff)/center_position
            
            motor.left( power )
        pass
    
        sleep( 0.1 )
    pass

pass


from machine import Pin, Timer
from time import sleep
import random

from picogo.Robot import Robot
from picogo.UltraSonic import UltraSonic 

def run_obstacle_avoidance( robot ) :
    duration = robot.duration
    speed = robot.speed
    max_dist = robot.max_dist
    
    left_block, right_block = robot.read_blocks()
    
    if not left_block and not right_block :
        dist = robot.obstacle_distance()
    else :
        dist = 2*max_dist 
    pass
    
    if left_block and right_block :
        robot.backward( speed )
    elif left_block :   # 좌측 장애물시, 우회전
        robot.right( speed ) 
    elif right_block :  # 우측 장애물시, 좌회전
        robot.left( speed ) 
    elif dist < max_dist : # 전방 장애물 있을 경우, 좌회전 
        robot.left( speed ) 
    else :              # 장매물이 없으면, 전진
        robot.forward( speed ) 
    pass 

    return left_block, right_block, ( dist < max_dist ), duration
pass ## -- run_ultrasonic_avoidance
    
def mainImpl( robot ) :
    print( f"mainImpl" ) 
    
    left_block = 0
    right_block = 0  
    dist = 0
    
    robot.disp_info_rects()
        
    while robot.run_ext_module :
        robot.disp_battery()
        robot.disp_motor() 
    
        [ left_block, right_block, is_obstacle, duration ] = run_obstacle_avoidance( robot )
        
        if left_block or right_block or is_obstacle :
            dur_count = random.randint( 30, 50 )
            sleep( dur_count*duration )
        else :
            sleep( duration )
        pass
    pass

    robot.stop()
    robot.disp_logo()

    print( f"Finished running ultrasonic ir avoidance." )

pass ## -- main

def main( robot ) :
    robot.run_ext_module = True
    
    import _thread 
    
    callback = lambda : mainImpl( robot=robot )
    
    _thread.start_new_thread( callback, () )
    
    print( "thread inited" )
pass

if __name__ is '__main__' :
    
    print( "Hello ..." )
        
    robot = Robot()
    main( robot )
    
    duration = 30
    print( f"sleep({duration})" )
    sleep( duration )
    print( f"finished sleep({duration})" )
    
    robot.run_ext_module = False
    
    print( "Good bye!" )
    
pass



from machine import Pin
from time import sleep_us

from .Motor import Motor

def getkey( irRemoteCon ):
        
    if irRemoteCon.value() == 0 :
        count = 0
        
        while (irRemoteCon.value() == 0) and (count < 100) : #9ms
            count += 1
            sleep_us(100)
        pass
        
        if(count < 10):
            return None
        pass
    
        count = 0
        
        while (irRemoteCon.value() == 1) and (count < 50) : #4.5ms
            count += 1
            sleep_us(100)
        pass
            
        idx = 0
        cnt = 0
        data = [0,0,0,0]
        
        for i in range(0,32) :
            count = 0
            while (irRemoteCon.value() == 0) and (count < 10) :    #0.56ms
                count += 1
                sleep_us(100)

            count = 0
            while (irRemoteCon.value() == 1) and (count < 20) :   #0: 0.56mx
                count += 1                                #1: 1.69ms
                sleep_us(100)

            if count > 7 :
                data[idx] |= 1<<cnt
            pass
        
            if cnt == 7 :
                cnt = 0
                idx += 1
            else:
                cnt += 1
            pass

        if data[0]+data[1] == 0xFF and data[2]+data[3] == 0xFF :  #check
            return data[2]
        else:
            return("repeat")
        pass
    
    pass

pass

if __name__=='__main__':
    
    motor = Motor()
    irRemoteCon = Pin(5, Pin.IN)
    speed = 20
    
    print( "Hello ... Remote Controller !" )
    
    while True:
        key = getkey( irRemoteCon )
        
        if(key != None):
            if key == 0x18:
                motor.forward(speed)
            elif key == 0x08:
                motor.left(20)
            elif key == 0x1c:
                motor.stop()
            elif key == 0x5a:
                motor.right(20)
            elif key == 0x52:
                motor.backward(speed)
            elif key == 0x09:
                speed = 50
                print(speed)
            elif key == 0x15:
                if(speed + 10 < 101):
                    speed += 10
                print(speed)
            elif key == 0x07:
                if(speed - 10 > -1):
                    speed -= 10
                print(speed)
            pass
        pass
    pass

pass



import time
    
def current_time_mili():
    return round(time.time() * 1000)
pass

def curr_time_mili():
    return current_time_mili()
pass

if __name__== '__main__' :
    print( 'Hello World!' )

    print( f"curr time = {current_time_mili()}" )
    
    time.sleep( 1 )
    
    print( f"curr time = {curr_time_mili()}" )
    
pass


from time import sleep

from picogo.Battery import Battery
from picogo.LCD import LCD

if __name__ == '__main__' :
    
    lcd = LCD()
    
    lcd.fill(0xF232)
    lcd.line(2,2,70,2,0xBB56)
    lcd.line(70,2,85,17,0xBB56)
    lcd.line(85,17,222,17,0xBB56)
    lcd.line(222,17,237,32,0xBB56)
    lcd.line(2,2,2,118,0xBB56)
    lcd.line(2,118,17,132,0xBB56)
    lcd.line(17,132,237,132,0xBB56)
    lcd.line(237,32,237,132,0xBB56)

    lcd.text("Raspberry Pi Pico", 90, 7, 0xFF00)
    lcd.text("PicoGo", 10, 7, 0x001F)
    lcd.text("Waveshare.com", 70, 120, 0x07E0)
    lcd.show()

    battery = Battery()
    
    count = 0
    while True :
        count += 1
        
        temperature, voltage, percent = battery.read()
        
        print( f"[{count:03d}] Temperature = {temperature:.3f} °C, Voltage = {voltage:.3f} V, Percent = {percent:.2} %" )
        
        y = 35
        lcd.fill_rect( 30, y, 160, 80, 0xF232 )
        
        lcd.text( f"count : {count}", 30, y, 0xFFFF ); y += 15
        lcd.text( f"temperature : {temperature:5.2f} °C", 30, y, 0xFFFF ); y += 15
        lcd.text( f"Voltage     : {voltage:5.2f} V", 30, y, 0xFFFF ); y += 15
        lcd.text( f"percent     : {percent:3.1f} %", 30, y, 0xFFFF ); y += 15

        lcd.show()
        sleep( 1 )
    pass

pass


from machine import Pin

class IRSensor :
    
    def __init__( self ) :
        self.ds_right = Pin(2, Pin.IN)
        self.ds_left  = Pin(3, Pin.IN)
    pass

    def read_blocks( self ) :
        return [ self.ds_left.value() == 0, self.ds_right.value() == 0 ]   # left_block, right_block
    pass

pass


import machine
import ujson, utime

from picogo.Robot import Robot

class BlueTooth :
    def __init__( self ):
        print( "Hello ... Bluetooth!" )
        
        self.robot = Robot()
        self.uart = self.robot.uart
        
        self.init_bluetooth()
    pass

    def init_bluetooth( self ) :
        robot = self.robot
        
        rgbLed = robot.rgbLed
        color = ( 0, 0, 0 )
        rgbLed.set_color( color )
                
    pass

    def send_reply( self, request_no, reply ) :
        uart = self.uart
        
        reply = f"{request_no}:{reply}\n"
        
        uart.write( reply )
        
        if not uart.txdone() :
            uart.flush()
        pass
        
        reply_len = len( reply )
        reply = reply.replace( "\n", "\\n" )
        
        print( f"send request_no: {request_no}, reply : '{reply}', len : {reply_len}" )
    pass

    def process_json_cmd( self, s, robot ) :
        reply = None
        
        try :
            reply = self.process_json_cmd_impl( s, robot )
        except Exception as e :
            print( e ) 
        pass
    
        return reply
    pass
    
    def process_json_cmd_impl( self, json, robot ) :
        reply = None
        
        uart = robot.uart
        
        motor = robot.motor
        rgbLed = robot.rgbLed
        buzzer = robot.buzzer
        led = robot.led
        
        speed = robot.speed
        
        cmd = json.get("Forward")
        
        if cmd != None:
            if cmd == "Down":
                motor.forward(speed, verbose=True)
                reply = "ok"
            elif cmd == "Up":
                motor.stop()
                reply = "ok"
            pass
        pass
                
        cmd = json.get("Backward")
        if cmd != None:
            if cmd == "Down":
                motor.backward(speed)
                reply = "ok"
            elif cmd == "Up":
                motor.stop()
                reply = "ok"
            pass
        pass
         
        cmd = json.get("Left")
        if cmd != None:
            if cmd == "Down":
                motor.left(20)
                reply = "ok"
            elif cmd == "Up":
                motor.stop()
                reply = "ok"
            pass
        pass
                 
        cmd = json.get("Right")
        if cmd != None:
            if cmd == "Down":
                motor.right(20)
                reply = "ok"
            elif cmd == "Up":
                motor.stop()
                reply = "ok"
            pass
        pass
      
        cmd = json.get("Low")
        if cmd == "Down":
            robot.speed = robot.low_speed
            reply = "ok"
        pass

        cmd = json.get("Medium")
        if cmd == "Down":
            robot.speed = robot.med_speed
            reply = "ok"
        pass

        cmd = json.get("High")
        if cmd == "Down":
            robot.speed = robot.high_speed
            reply = "ok"
        pass
        
        cmd = json.get("BZ")
        if cmd != None:
            if cmd == "on":
                buzzer.value(1)
                reply = "ok"
            elif cmd == "off":
                buzzer.value(0)
                reply = "ok"
            pass
        pass
        
        cmd = json.get("LED")
        if cmd != None:
            if cmd == "on":
                led.value(1)
                reply = "ok"
            elif cmd == "off":
                led.value(0)
                reply = "ok"
            pass
        pass
        
        cmd = json.get("RGB")
        if cmd != None:
            rgb = tuple( eval(cmd) )
            
            rgbLed.set_color( rgb )
            
            reply = "ok"
        pass
    
        if reply is None :
            reply = "bad"
        pass
    
        return reply
    pass

    def process_text_cmd( self, s, robot ) :
        print( f"process_text_cmd s = [{s}]" )
        reply = None
        
        try :
            reply = self.process_text_cmd_impl( s, robot )
        except Exception as e :
            print( e )
        pass
    
        return reply
    pass

    def process_text_cmd_impl( self, s, robot ) :
        reply = None
        
        if "hello" in s :
            reply = "ok"
        elif "send me pairing code" in s :
            print( f"processing pairing code" )
            
            pair_code = robot.lcd.disp_pairing_code( flush=True )
            
            print( f"pair code = {pair_code}" )
            
            reply = f"pair code: {pair_code}"  
        elif "paring completed" in s :
            print( f"paring completed" )
            
            robot.lcd.disp_logo()

            reply = f"ok"
        elif "start obstacle avoidance" in s :
            robot.beepOnOff( repeat=2, period=0.3 )
            
            from picogo import ObstacleAvoidance
            
            ObstacleAvoidance.main( robot )
        elif "start lane following" in s :
            robot.beepOnOff( repeat=2, period=0.3 )
            
            from picogo import LineTrackingPID
            
            LineTrackingPID.main( robot )
        elif "speed" in s:
            print( f"speed" )
            
            value = s.split( "=" )[-1]
            speed = robow.low_speed
            
            try : 
                speed= int( value.strip() )
            except:
                pass
            pass
            
            robot.speed = speed
            
            reply = f"ok" 
        elif "stop" in s:
            print( f"stop" )
            
            robot.run_ext_module = False
            robot.stop()
            
            robot.beepOnOff( repeat=3, period=0.3 )
            
            robot.lcd.disp_logo()
            
            reply = f"ok"
        elif s in "good bye" or "disconnect" in s:
            print( f"good bye" )
            
            robot.run_ext_module = False
            robot.stop()
            
            robot.beepOnOff( repeat=4, period=0.3 )
            
            robot.lcd.disp_version()
            
            reply = f"ok"
        else :
            print( f"Unknown command = [{s}]" )
            
            reply = "Unknown code"
        pass
    
        return reply
    pass

    def read_bytes( self, uart, nbytes ) :
        byte_list = bytearray( nbytes )
            
        i = 0 
        while i < nbytes : 
            b = uart.read( 1 )
            if b :
                byte_list[ i ] = b[ 0 ]
                i += 1
            pass
        pass

        if nbytes == 1 :
            return byte_list[ 0 ]
        else :
            return byte_list
        pass
    pass
            
    def main( self ) :
        print( "Ready to accept!" )
        
        t = 0
        count = 0 

        robot = self.robot

        uart = robot.uart

        head_read_count = 0
        
        while True :
            debug = True
            
            data_type = 0
            data_len  = 0
                
            # read data type and data len
            while data_type == 0 :
                head_read_count += 1
                
                request_no = 0
                
                debug and print( f"[{head_read_count:04d}] reading sart of heading ..." )
                start_of_heading = self.read_bytes( uart, 1 )
                debug and print( f"[{head_read_count:04d}] start of heading = {start_of_heading}" )
                
                if start_of_heading != 1 :
                    data_type = 0
                    data_len  = 0
                elif start_of_heading == 1 : # start of heading
                    # read read request_no
                    if True : 
                        byte_list = self.read_bytes( uart, 2 )
                        
                        request_no = 0
                        for b in byte_list :
                            request_no = data_len*255 + b
                        pass
                    pass
                
                    # read data type
                    data_type = self.read_bytes( uart, 1 ) 
                    
                    # read data_len
                    if data_type : 
                        byte_list = self.read_bytes( uart, 2 )
                        
                        data_len = 0
                        for b in byte_list :
                            data_len = data_len*255 + b
                        pass
                    pass
                pass
            pass

            print( f"request_no = {request_no}, date_type = {data_type}, data_len = {data_len}" )

            # reate data
            data = None
            
            if data_len > 0 :
                data = self.read_bytes( uart, data_len )  
            else :
                data = None
            pass

            # read end of transmission
            if data != None : 
                end_of_transmission = self.read_bytes( uart, 1 )  
            
                if end_of_transmission != 4 : # End of Transmission
                    data = None
                pass 
            pass

            if data : 
                s = data.decode()
            pass
            
            if s != None :
                count += 1
                
                print( f"[{count:04d}] s = [{s}]" )                
                
                reply = None
                
                json = None
                    
                try : 
                    json = ujson.loads(s)
                except Exception as e:
                    #print( e )
                    pass
                pass
    
                if json != None :
                    print( "json =" , json )
    
                    reply = self.process_json_cmd( json, robot )
                elif s != None :
                    reply = self.process_text_cmd( s, robot )
                pass
            
                if reply is None :
                    reply = "Unknown code"
                pass
            
                if reply != None :
                    self.send_reply( request_no, reply )
                pass
            pass
            
            if s != None and (utime.ticks_ms() - t) > 3000 :
                showMsg = False
                if showMsg :
                    t = utime.ticks_ms()
                    
                    text = f"PicoGo\n{count}"
                    
                    robot.lcd.disp_full_text( text )
                    robot.lcd.show()
                pass
            pass

        pass
    pass

pass

def main() :
    blueTooth = BlueTooth()
    blueTooth.main()
pass

if __name__ == '__main__' :
    main()
pass
    


from picogo.LCD import LCD
        
if __name__=='__main__':
    lcd = LCD()
    
    width = lcd.width
    height = lcd.height
    
    lcd.fill( lcd.WHITE )
    
    x = 10; y = 10;  w = width
    
    # 선 그리기 
    lcd.line( x, y, x + w - 2*x - 1 , y, lcd.BLUE )
    
    # 사각형 채우기    
    y = 20 ; h = 20
    lcd.rect( x, y, w - 2*x - 1 , h, lcd.YELLOW, True )
    
    # 사각형 외곽선 그리기 
    y += h + 10 ; h = 20
    lcd.rect( x, y, w - 2*x - 1 , h, lcd.RED, False )
    
    # 타원 그리기 
    y += 2*h + 10 ; w = 50
    lcd.ellipse( x + w, y, w, h, lcd.BLUE, True ) 
        
    lcd.show() 
        
pass


from picogo.LCD import LCD 
        
if __name__=='__main__':
    # LCDPrintTest.py
    
    lcd = LCD() 
    
    lcd.print( "Hello...." )
    lcd.print( "Raspberry Pi Pico", c=LCD.BLUE )
    lcd.print( "PicoGo", c=LCD.WHITE ) 
    
    lcd.print( "Hello...." )
    lcd.print( "Raspberry Pi Pico", c=LCD.BLUE )
    lcd.print( "PicoGo", c=LCD.WHITE )
    lcd.print( "Waveshare.com", c=LCD.RED ) 
    
    lcd.show()
pass


from setuptools import setup

__project__ = 'picogo'
__packages__ = ['picogo']
__desc__ = 'A beginner-friendly library for using picogo with the Raspberry Pi Pico. '
__version__ = '0.0.22'
__author__ = "SkySLAM Co., Ltd."
__author_email__ = 'terabuilder@gmail.com'
__license__ = 'MIT'
__url__ = 'https://github.com/sunabove/PicoGo'
__keywords__ = [
    'picogo',
    'pico',
    'raspberry',
    'raspberry pi',
    'picorun',
    'skyslam',
]
__classifiers__ = [
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Programming Language :: Python :: Implementation :: MicroPython',
    ]
__long_description__ = """A beginner-friendly library for using picogo with the Raspberry Pi Pico.

```python
from picogo import Robot

robot = Robot()
robot.forward( speed = 30 )

```

Documentation is available at (https://github.com/sunabove/PicoGo/).
"""

setup(
    name=__project__,
    version=__version__,
    description=__desc__,
    long_description=__long_description__,
    long_description_content_type='text/markdown',
    url=__url__,
    author=__author__,
    author_email=__author_email__,
    license=__license__,
    classifiers=__classifiers__,
    keywords=__keywords__,
    packages=__packages__, 
)


if __name__ is '__main__' :
    
    from machine import Pin, Timer
    
    led = Pin(25, Pin.OUT)
    
    timer = Timer() 

    class Blink :
        def __init__( self, max_count = 10 ) :
            self.count = 0
            self.max_count = max_count
            
        def blink( self, timer ):
            self.count += 1
                
            if self.count <= self.max_count :
                print( f"[{self.count}] toggle" )
                led.toggle()
            else :
                print( "timer deinit" )
                timer.deinit()
            pass
        pass
    pass

    blink = Blink()

    timer.init( freq=2, mode=Timer.PERIODIC, callback=blink.blink )
    
pass



from time import sleep

from picogo.Robot import Robot
from picogo.UltraSonic import UltraSonic

def main( robot = None ) :
    if robot is None : robot = Robot()
    
    ultraSonic = robot.ultraSonic
    
    duration = 0.02
    speed = 20
    max_dist = 18
    
    obstacle_cnt = 0
    
    while True :
        dist = ultraSonic.distance()
        
        print( f"distance = {dist:6.2f} cm" )
        
        if dist < max_dist :
            obstacle_cnt += 1
            
            if obstacle_cnt == 1 : 
                robot.stop()
                sleep( duration )
            pass
            
            robot.right( 2*speed )
        else:
            ostacle_cnt = 0 
            robot.forward( speed )
        pass
            
        sleep( duration )
    pass 
pass

if __name__ is '__main__' :
    
    print( "Hello ..." )
        
    robot = Robot()
    main( robot )
    
pass
    
    


from time import sleep 

from picogo.LCD import LCD
from picogo.UltraSonic import UltraSonic
        
if __name__=='__main__': 
    
    lcd = LCD()
    ultraSonic = UltraSonic()
    
    lcd.disp_init()
    
    while 1 :
        dist = ultraSonic.obstacle_distance()
        lcd.disp_ultra_sonic( dist )
        lcd.show()
        
        print( f"Distance = {dist:6.2f} cm" )
        sleep( 0.1)
    pass
pass


def copy() :
    print( "main_file_copy" )

    file_path = "/main.py"
    
    file_read = None 
    
    try : 
        main_file_path = "/lib/picogo/main.py" 
        
        print( f"main file path = {main_file_path}" )
        
        file_read = open( main_file_path, "r" )
    except Exception as e:
        print( f"cannot find main file : {main_file_path}" )
        
        file_read = None
    pass

    if file_read is None :
        try : 
            main_file_path = "/picogo/main.py" 
            
            print( f"main file path = {main_file_path}" )
            
            file_read = open( main_file_path, "r" )
        except Exception as e:
            print( f"cannot find main file : {main_file_path}" )
            
            file_read = None
        pass
    pass

    if file_read is None :
        print( "cannot find main file" )
        print( "Filed to copy main file." )
    elif file_read is not None :
        file = open( file_path, "w" ) 
        
        for line in file_read.readlines() :
            file.write( line )
        pass
                    
        file_read.close()
        file.close()
        
        print( "Success main file copy" )
    pass
pass

copy()


if __name__== '__main__' :
    
    print( 'Hello PicoGo!' )
    print( 'Running Bluetooth program ... ' )
    
    from picogo import BlueTooth
    
    BlueTooth.main()
    
    print( 'Good bye!' )
    
pass